<?php

namespace App\Imports;

use App\Models\Product;
use App\Models\Unit;
use Maatwebsite\Excel\Concerns\ToModel;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\SkipsOnFailure;
use Maatwebsite\Excel\Concerns\SkipsFailures;
use Illuminate\Support\Facades\Validator;

class ProductsImport implements ToModel, WithHeadingRow
{
    use SkipsFailures;

    protected $units;
    protected $existingNames;
    protected $existingBarcodes;
    protected $processedNames;
    protected $processedBarcodes;
    public $successfulRows = [];
    public $failedRows = [];

    public function __construct()
    {
        $this->units = Unit::pluck('id')->toArray();
        $this->existingNames = Product::pluck('name')->toArray();
        $this->existingBarcodes = Product::pluck('code')->toArray();
        $this->processedNames = [];
        $this->processedBarcodes = [];
    }

    public function model(array $row)
    {
        $validator = Validator::make($row, [
            'satuan_dasar' => 'required|in:' . implode(',', $this->units),
            'kategori' => 'required|in:product,service',
            'nama_produk' => 'required|unique:products,name',
            'barcode' => 'nullable|unique:products,code',
        ]);

        if ($validator->fails()) {
            $this->failedRows[] = ['row' => $row, 'errors' => $validator->errors()->all()];
            return null;
        }

        // Additional custom validation
        if (in_array(strtolower($row['nama_produk']), $this->processedNames)) {
            $this->failedRows[] = ['row' => $row, 'errors' => ['Duplicate product name in file']];
            return null;
        }

        if (isset($row['barcode']) && in_array($row['barcode'], $this->processedBarcodes)) {
            $this->failedRows[] = ['row' => $row, 'errors' => ['Duplicate barcode in file']];
            return null;
        }

        $this->processedNames[] = strtolower($row['nama_produk']);
        if (isset($row['barcode'])) {
            $this->processedBarcodes[] = $row['barcode'];
        }

        $this->successfulRows[] = $row;

        return new Product([
            'name' => $row['nama_produk'],
            'unit' => $row['satuan_dasar'],
            'purchase_price' => $row['harga_beli'] ?? 0,
            'selling_price' => $row['harga_jual'],
            'producttype' => $row['jenis_produk'] ?? 1,
            'code' => $row['barcode'] ?? null,
            'category' => $row['kategori'],
            'qty' => 0,
            'product_image' => null,
            'display_rack' => $row['etalase'] ?? null,
            'qty_minimum' => $row['jumlah_minimum'] ?? 0,
            'for_sale' => $row['apakah_dibeli'] ?? 1,
            'for_purchase' => $row['apakah_dijual'] ?? 1,
        ]);
    }

    public function getResults()
    {
        return [
            'successfulRows' => $this->successfulRows,
            'failedRows' => $this->failedRows,
        ];
    }
}
